@@ -187,20 +187,6 @@ reload_xinerama_infos (MetaScreen *screen)
 {
   MetaDisplay *display;
 
-  {
-    GList *tmp;
-
-    tmp = screen->workspaces;
-    while (tmp != NULL)
-      {
-        MetaWorkspace *space = tmp->data;
-
-        meta_workspace_invalidate_work_area (space);
-        
-        tmp = tmp->next;
-      }
-  }
-
   display = screen->display;
   
   if (screen->xinerama_infos)
@@ -362,6 +348,20 @@ reload_xinerama_infos (MetaScreen *screen)
 
   g_assert (screen->n_xinerama_infos > 0);
   g_assert (screen->xinerama_infos != NULL);
+
+  {
+    GList *tmp;
+
+    tmp = screen->workspaces;
+    while (tmp != NULL)
+      {
+        MetaWorkspace *space = tmp->data;
+
+        meta_workspace_invalidate_work_area (space);
+        
+        tmp = tmp->next;
+      }
+  }
 }
 
 MetaScreen*
@@ -665,8 +665,11 @@ meta_screen_free (MetaScreen *screen)
   
   meta_display_grab (display);
 
-  meta_compositor_unmanage_screen (screen->display->compositor,
-                                   screen);
+  if (screen->display->compositor)
+    {
+      meta_compositor_unmanage_screen (screen->display->compositor,
+				       screen);
+    }
   
   meta_display_unmanage_windows_for_screen (display, screen);
   
@@ -731,62 +734,104 @@ meta_screen_free (MetaScreen *screen)
   meta_display_ungrab (display);
 }
 
-void
-meta_screen_manage_all_windows (MetaScreen *screen)
+typedef struct
+{
+  Window		xwindow;
+  XWindowAttributes	attrs;
+} WindowInfo;
+
+static GList *
+list_windows (MetaScreen *screen)
 {
   Window ignored1, ignored2;
   Window *children;
-  int n_children;
-  int i;
+  guint n_children, i;
+  GList *result;
 
-  /* Must grab server to avoid obvious race condition */
-  meta_display_grab (screen->display);
-
-  meta_error_trap_push_with_return (screen->display);
-  
   XQueryTree (screen->display->xdisplay,
               screen->xroot,
               &ignored1, &ignored2, &children, &n_children);
 
-  if (meta_error_trap_pop_with_return (screen->display, TRUE) != Success)
+  result = NULL;
+  for (i = 0; i < n_children; ++i)
     {
-      meta_display_ungrab (screen->display);
-      return;
-    }
+      WindowInfo *info = g_new0 (WindowInfo, 1);
 
-  meta_stack_freeze (screen->stack);
-  i = 0;
-  while (i < n_children)
-    {
-      XWindowAttributes attrs;
-      
       meta_error_trap_push_with_return (screen->display);
       
       XGetWindowAttributes (screen->display->xdisplay,
-                            children[i], &attrs);
-      
-      if (meta_error_trap_pop_with_return (screen->display, TRUE) != Success)
-        {
+                            children[i], &info->attrs);
+
+      if (meta_error_trap_pop_with_return (screen->display, TRUE))
+	{
           meta_verbose ("Failed to get attributes for window 0x%lx\n",
                         children[i]);
+	  g_free (info);
         }
       else
         {
-          meta_window_new_with_attrs (screen->display, children[i], TRUE,
-                                      &attrs);
+	  info->xwindow = children[i];
+	}
 
-          meta_compositor_add_window (screen->display->compositor,
-                                      children[i], &attrs);
-        }
+      result = g_list_prepend (result, info);
+    }
 
-      ++i;
+  if (children)
+    XFree (children);
+
+  return g_list_reverse (result);
+}
+
+void
+meta_screen_manage_all_windows (MetaScreen *screen)
+{
+  GList *windows;
+  GList *list;
+
+  meta_display_grab (screen->display);
+  
+  windows = list_windows (screen);
+
+  meta_stack_freeze (screen->stack);
+  for (list = windows; list != NULL; list = list->next)
+    {
+      WindowInfo *info = list->data;
+
+      meta_window_new_with_attrs (screen->display, info->xwindow, TRUE,
+				  &info->attrs);
     }
   meta_stack_thaw (screen->stack);
 
+  g_list_foreach (windows, (GFunc)g_free, NULL);
+  g_list_free (windows);
+
   meta_display_ungrab (screen->display);
-  
-  if (children)
-    XFree (children);
+}
+
+void
+meta_screen_composite_all_windows (MetaScreen *screen)
+{
+  GList *windows, *list;
+
+  if (!screen->display->compositor)
+    return;
+
+  windows = list_windows (screen);
+
+  meta_stack_freeze (screen->stack);
+
+  for (list = windows; list != NULL; list = list->next)
+    {
+      WindowInfo *info = list->data;
+
+      meta_compositor_add_window (screen->display->compositor,
+				  info->xwindow, &info->attrs);
+    }
+
+  meta_stack_thaw (screen->stack);
+
+  g_list_foreach (windows, (GFunc)g_free, NULL);
+  g_list_free (windows);
 }
 
 MetaScreen*
@@ -1715,7 +1760,7 @@ set_workspace_names (MetaScreen *screen)
                    screen->display->atom_net_desktop_names,
 		   screen->display->atom_utf8_string,
                    8, PropModeReplace,
-		   flattened->str, flattened->len);
+		   (unsigned char *)flattened->str, flattened->len);
   meta_error_trap_pop (screen->display, FALSE);
   
   g_string_free (flattened, TRUE);
